@@ -25,16 +25,14 @@ use strict;
 use warnings;
 use re 'taint';
 
-BEGIN {
-  use Exporter ();
-  use vars qw(@ISA);
-  @ISA = qw(Exporter);
+use Exporter ();
+our  @ISA = qw(Exporter);
 
-  use vars qw (
-	@BAYES_VARS @IP_VARS @SA_VARS %EXPORT_TAGS @EXPORT_OK
-  );
+our(@BAYES_VARS, @IP_VARS, @SA_VARS, %EXPORT_TAGS, @EXPORT_OK);
 
-  @IP_VARS = qw(
+# NOTE: Unless you need these to be available at BEGIN time, you're better with this out of a BEGIN block with a simple our statement.
+BEGIN { 
+    @IP_VARS = qw(
 	IP_IN_RESERVED_RANGE IP_PRIVATE LOCALHOST IPV4_ADDRESS IP_ADDRESS
   );
   @BAYES_VARS = qw(
@@ -253,10 +251,10 @@ use constant LOCALHOST => qr/
 # an IP address, in IPv4 format only.
 #
 use constant IPV4_ADDRESS => qr/\b
-		    (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-                    (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-                    (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-                    (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)
+		    (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+                    (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+                    (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+                    (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)
                   \b/ox;
 
 # ---------------------------------------------------------------------------
@@ -267,10 +265,10 @@ use constant IP_ADDRESS => qr/
 		    (?:
 		      \b(?<!:)	# ensure no "::" IPv4 marker before this one
 		      # plain IPv4, as above
-		      (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-		      (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-		      (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-		      (?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\b
+		      (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+		      (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+		      (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+		      (?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\b
 		    |
 		      # IPv6 addresses
 		      # don't use \b here, it hits on :'s
@@ -303,10 +301,10 @@ use constant IP_ADDRESS => qr/
 			  (?:[a-f0-9]{1,4}:){4}:[a-f0-9]{1,4}:
 			)
 			# and the IPv4 address appended to all of the 12 bytes above
-			(?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-			(?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-			(?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)\.
-			(?:1\d\d|2[0-4]\d|25[0-5]|\d\d|\d)   # no \b, we check later
+			(?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+			(?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+			(?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)\.
+			(?:1\d\d|2[0-4]\d|25[0-5]|[1-9]\d|\d)   # no \b, we check later
 
 			| # or (separately) a pure IPv6 address
 
@@ -401,7 +399,7 @@ use constant AI_TIME_UNKNOWN => 0;
 
 # Charsets which use capital letters heavily in their encoded representation.
 use constant CHARSETS_LIKELY_TO_FP_AS_CAPS => qr{[-_a-z0-9]*(?:
-	  koi|jp|jis|euc|gb|big5|isoir|cp1251|georgianps|pt154|tis
+	  koi|jp|jis|euc|gb|big5|isoir|cp1251|windows-1251|georgianps|pt154|tis
 	)[-_a-z0-9]*}ix;
 
 1;
