@@ -37,18 +37,17 @@
 
 # ---------------------------------------------------------------------------
 
-package Mail::SpamAssassin::Message::Metadata::Received;
-1;
+use strict;  # make Test::Perl::Critic happy
+package Mail::SpamAssassin::Message::Metadata::Received; 1;
 
 package Mail::SpamAssassin::Message::Metadata;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
 use Mail::SpamAssassin::Dns;
 use Mail::SpamAssassin::PerMsgStatus;
-use Mail::SpamAssassin::Util::RegistrarBoundaries;
 use Mail::SpamAssassin::Constants qw(:ip);
 
 # ---------------------------------------------------------------------------
@@ -56,6 +55,14 @@ use Mail::SpamAssassin::Constants qw(:ip);
 sub parse_received_headers {
   my ($self, $permsgstatus, $msg) = @_;
 
+  my $suppl_attrib = $msg->{suppl_attrib};  # out-of-band info from a caller
+
+  # a caller may assert that a message is coming from inside or from an
+  # authenticated roaming users; this info may not be available in mail
+  # header section, e.g. in case of nonstandard authentication mechanisms
+  my $originating;  # boolean
+  $originating = $suppl_attrib->{originating}  if ref $suppl_attrib;
+
   $self->{relays_trusted} = [ ];
   $self->{num_relays_trusted} = 0;
   $self->{relays_trusted_str} = '';
@@ -77,6 +84,8 @@ sub parse_received_headers {
   $self->{last_trusted_relay_index} = -1;	# last counting from the top,
   $self->{last_internal_relay_index} = -1;	# first in time
 
+  $self->{allow_mailfetch_markers} = 1;         # This needs to be set for the
+                                                # first Received: header
   # now figure out what relays are trusted...
   my $trusted = $permsgstatus->{main}->{conf}->{trusted_networks};
   my $internal = $permsgstatus->{main}->{conf}->{internal_networks};
@@ -190,7 +199,15 @@ sub parse_received_headers {
       } else {
 	# trusted_networks matches?
 	if (!$relay->{auth} && !$trusted->contains_ip($relay->{ip})) {
-	  $in_trusted = 0;
+	  if (!$originating) {
+	    $in_trusted = 0;	# break the trust chain
+	  } else {  # caller asserts a msg was submitted from inside or auth'd
+	    $found_msa = 1;	# let's assume the previous hop was actually
+				# an MSA, and propagate trust from here on
+	    dbg('received-header: originating, '.
+	        '%s and remaining relays will be considered trusted%s',
+	        $relay->{ip}, !$in_internal ? '' : ', but no longer internal');
+	  }
 	  $in_internal = 0;	# if it's not trusted it's not internal
 	} else {
 	  # internal_networks matches?
@@ -223,11 +240,11 @@ sub parse_received_headers {
 
     if ($in_trusted) {
       push (@{$self->{relays_trusted}}, $relay);
-      $self->{allow_fetchmail_markers} = 1;
+      $self->{allow_mailfetch_markers} = 1;
       $self->{last_trusted_relay_index}++;
     } else {
       push (@{$self->{relays_untrusted}}, $relay);
-      $self->{allow_fetchmail_markers} = 0;
+      $self->{allow_mailfetch_markers} = 0;
     }
 
     if ($in_internal) {
@@ -384,11 +401,15 @@ sub parse_received_line {
 
   # try to catch authenticated message identifier
   #
-  # with ESMTPA, ESMTPSA, LMTPA, LMTPSA should cover RFC 3848 compliant MTAs
+  # with ESMTPA, ESMTPSA, LMTPA, LMTPSA should cover RFC 3848 compliant MTAs,
+  # UTF8SMTPA and UTF8LMTPA are covered by RFC 4954 and RFC 6531,
   # with ASMTP (Authenticated SMTP) is used by Earthlink, Exim 4.34, and others
   # with HTTP should only be authenticated webmail sessions
   # with HTTPU is used by Communigate Pro with Pronto! webmail interface
-  if (/ by / && / with (ESMTPA|ESMTPSA|LMTPA|LMTPSA|ASMTP|HTTPU?)(?: |$)/i) {
+  # with HTTPS is used by Horde adjusts the Received header to say "HTTPS" when
+  # a connection is made over HTTPS
+  # IANA registry: http://www.iana.org/assignments/mail-parameters/mail-parameters.xhtml
+  if (/ by / && / with ((?:ES|L|UTF8S|UTF8L)MTPS?A|ASMTP|HTTP[SU]?)(?: |;|$)/i) {
     $auth = $1;
   }
   # GMail should use ESMTPSA to indicate that it is in fact authenticated,
@@ -404,6 +425,10 @@ sub parse_received_line {
   elsif (/authenticated/ && /^from .*?(?:\](?: \([^)]*\))?\)|\)\]) .*?\(.*?authenticated.*?\).*? by/) {
     $auth = 'Sendmail';
   }
+  # workaround for GMX, which authenticates users but does not indicate it properly - # SMTP version
+  elsif (/from \S* \((?:HELO|EHLO) (\S*)\) \[(${IP_ADDRESS})\] by (mail\.gmx\.(?:net|com)) \([^\)]+\) with ((?:ESMTP|SMTP))/) {
+    $auth = "GMX ($4 / $3)";
+  }
   # Critical Path Messaging Server
   elsif (/ \(authenticated as /&&/\) by .+ \(\d{1,2}\.\d\.\d{3}(?:\.\d{1,3})?\) \(authenticated as .+\) id /) {
     $auth = 'CriticalPath';
@@ -412,14 +437,19 @@ sub parse_received_line {
   elsif (/\) \(Authenticated sender: \S+\) by \S+ \(Postfix\) with /) {
     $auth = 'Postfix';
   }
-  # Communigate Pro
-  elsif (/CommuniGate Pro SMTP/ && / \(account /) {
+  # Communigate Pro - Bug 6495 adds HTTP as possible transmission method
+  # 	Bug 7277: XIMSS used by Pronto and other custom apps, IMAP supports XMIT extension
+  elsif (/CommuniGate Pro (HTTP|SMTP|XIMSS|IMAP)/ && / \(account /) {
     $auth = 'Communigate';
   }
   # Microsoft Exchange (complete with syntax error)
   elsif (/ with Microsoft Exchange Server HTTP-DAV\b/) {
     $auth = 'HTTP-DAV';
   }
+  # froufrou mailers like United Internet use a '(via HTTP)' comment, Bug 7101
+  elsif (/ by / && / \(via (HTTP.?)\)(?: |;|$)/i) {
+    $auth = $1;
+  }
 
 # ---------------------------------------------------------------------------
 
@@ -429,12 +459,14 @@ sub parse_received_line {
       $envfrom = $1;
     }
 
-    # bug 3236: ignore Squirrelmail injection steps.
     # from 142.169.110.122 (SquirrelMail authenticated user synapse) by
     # mail.nomis80.org with HTTP; Sat, 3 Apr 2004 10:33:43 -0500 (EST)
-    if (/ \(SquirrelMail authenticated user /) {
-      dbg("received-header: ignored SquirrelMail injection: $_");
-      return 0;
+    # Expanded to NaSMail Bug 6783
+    if (/ \((?:SquirrelMail|NaSMail) authenticated user /) {
+      #REVERTING bug 3236 and implementing re: bug 6549
+      if (/(${IP_ADDRESS})\b(?![.-]).{10,80}by (\S+) with HTTP/) {
+        $ip = $1; $by = $2; goto enough;
+      }
     }
 
     # AOL WebMail headers
@@ -690,6 +722,11 @@ sub parse_received_line {
       $rdns = $1; $helo = $2; $ip = $3; $by = $4; goto enough;
     }
 
+    # Received: from mail-backend.DDDD.com (LHLO mail-backend.DDDD.com) (10.2.2.20) by mail-backend.DDDD.com with LMTP; Thu, 18 Jun 2015 16:50:56 -0700 (PDT)
+    if (/^(\S+) \(LHLO (\S*)\) \((${IP_ADDRESS})\) by (\S+) with LMTP/) {
+      $rdns = $1; $helo = $2; $ip = $3; $by = $4; goto enough;
+    }
+
     # from dslb-082-083-045-064.pools.arcor-ip.net (EHLO homepc) [82.83.45.64] by mail.gmx.net (mp010) with SMTP; 03 Feb 2007 13:13:47 +0100
     if (/^(\S+) \((?:HELO|EHLO) (\S*)\) \[(${IP_ADDRESS})\] by (\S+) \([^\)]+\) with (?:ESMTP|SMTP)/) {
       $rdns = $1; $helo = $2; $ip = $3; $by = $4; goto enough;
@@ -840,12 +877,19 @@ sub parse_received_line {
       $helo = $1; $rdns = $2; $ip = $3; $by = $4; goto enough;
     }
 
+    # Identify fetch-from-server incidents:
+    # Fetchmail:
     # Received: from cabbage.jmason.org [127.0.0.1]
     # by localhost with IMAP (fetchmail-5.9.0)
     # for jm@localhost (single-drop); Thu, 13 Mar 2003 20:39:56 -0800 (PST)
-    if (/fetchmail/&&/^(\S+) (?:\[(${IP_ADDRESS})\] )?by (\S+) with \S+ \(fetchmail/) {
+    #
+    # Getmail:
+    # Received: from pop3.mail.dk (195.41.46.251) by loki.valhalla with POP3;
+    # 14 Apr 2010 11:14:29 -0000
+    #
+    if (/with (?:POP3|IMAP)/) {
       $self->found_pop_fetcher_sig();
-      return 0;		# skip fetchmail handovers
+      return 0;		# skip mail fetcher handovers
     }
 
     # Let's try to support a few qmailish formats in one;
@@ -855,9 +899,14 @@ sub parse_received_line {
     if (/^(\S+) \((?:HELO|EHLO) ([^\)]*)\) \((\S*@)?\[?(${IP_ADDRESS})\]?\).* by (\S+) /)
     {
       $mta_looked_up_dns = 1;
-      $rdns = $1; $helo = $2; $ident = (defined $3) ? $3 : '';
-      $ip = $4; $by = $5;
-      if ($ident) { $ident =~ s/\@$//; }
+      $rdns = $1; 
+      $helo = $2; 
+      $ident = (defined $3) ? $3 : '';
+      $ip = $4; 
+      $by = $5;
+      if ($ident) { 
+        $ident =~ s/\@$//; 
+      }
       goto enough;
     }
 
@@ -878,14 +927,6 @@ sub parse_received_line {
       $ip = $1; $by = $2; goto enough;
     }
 
-    # Received: from po11.mit.edu [18.7.21.73]
-    # by stark.dyndns.tv with POP3 (fetchmail-5.9.7)
-    # for stark@localhost (single-drop); Tue, 18 Feb 2003 10:43:09 -0500 (EST)
-    # by po11.mit.edu (Cyrus v2.1.5) with LMTP; Tue, 18 Feb 2003 09:49:46 -0500
-    if (/ with POP3 /&&/^(\S+) \[(${IP_ADDRESS})\] by (\S+) with POP3 /) {
-      $rdns = $1; $ip = $2; $by = $3; goto enough;
-    }
-
     # Received: from snake.corp.yahoo.com(216.145.52.229) by x.x.org via smap (V1.3)
     # id xma093673; Wed, 26 Mar 03 20:43:24 -0600
     if (/ via smap /&&/^(\S+)\((${IP_ADDRESS})\) by (\S+) via smap /) {
@@ -1071,6 +1112,12 @@ sub parse_received_line {
       goto enough;
     }
 
+    # Yahoo Authenticated SMTP; Bug #6535
+    # from itrqtnlnq (lucilleskinner@93.124.107.183 with login) by smtp111.mail.ne1.yahoo.com with SMTP; 17 Jan 2011 08:23:27 -0800 PST
+    if (/^(\S+) \((\S+)@(${IP_ADDRESS}) with login\) by (\S+\.yahoo\.com) with SMTP/) {
+      $helo = $1; $ip = $3; $by = $4; goto enough;
+    }
+
     # a synthetic header, generated internally:
     # Received: X-Originating-IP: 1.2.3.4
     if (/^X-Originating-IP: (\S+)$/) {
@@ -1143,7 +1190,7 @@ sub parse_received_line {
     # logging a little more.
     if (/^\S+ by \S+ \(.{0,100}\) with qmail-scanner/) {
       $envfrom =~ s/^\s*<*//gs; $envfrom =~ s/>*\s*$//gs;
-      $envfrom =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
+      $envfrom =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
       $self->{qmail_scanner_env_from} = $envfrom; # hack!
       return 0;
     }
@@ -1152,6 +1199,11 @@ sub parse_received_line {
     # id 18tOsg-0008FX-00; Thu, 13 Mar 2003 09:20:06 +0000
     if (/^\S+ by \S+ with local/) { return 0; }
 
+    # Local unix socket handover from Cyrus, tested with v2.3.14
+    # Received: from testintranator.net.vm ([unix socket])_ by testintranator.net.vm (Cyrus v2.3.14) with LMTPA;_ Tue, 21 Jul 2009 14:34:14 +0200
+    # Attention: Actually the received header is parsed as "testintranator.net.vm ([unix socket]) by testintranator.net.vm (Cyrus v2.3.14) with LMTPA", "from" is ommited.
+    if (/^\S+ \(\[unix socket\]\) by \S+ \(Cyrus v[0-9]*?\.[0-9]*?\.[0-9]*?\) with LMTPA/) { return 0; }
+
     # HANDOVERS WE KNOW WE CAN'T DEAL WITH: TCP transmission, but to MTAs that
     # just don't log enough info for us to use (ie. no IP address present).
     # Note: "return 0" is strongly recommended here, unless you're sure
@@ -1255,12 +1307,15 @@ enough:
     $rdns = '';		# some MTAs seem to do this
   }
   
-  $ip =~ s/^\[//; $ip =~ s/\]$//;
+  $ip =~ s/^ipv6://i;   # remove "IPv6:" prefix
+  $ip =~ s/^\[//; $ip =~ s/\]\z//;
 
-  $ip =~ s/^ipv6://i;   # remove optional "IPv6:" prefix
+  # IPv6 Scoped Address (RFC 4007, RFC 6874, RFC 3986 "unreserved" charset)
+  $ip =~ s/%[A-Z0-9._~-]*\z//si;  # scoped address? remove <zone_id>
 
   # remove "::ffff:" prefix from IPv4-mapped-in-IPv6 addresses,
-  # so we can treat them as simply IPv4 addresses
+  # so we can treat them simply as IPv4 addresses
+  # (only handles 'alternative form', not 'preferred form' - to be improved)
   $ip =~ s/^0*:0*:(?:0*:)*ffff:(\d+\.\d+\.\d+\.\d+)$/$1/i;
 
   $envfrom =~ s/^\s*<*//gs; $envfrom =~ s/>*\s*$//gs;
@@ -1270,12 +1325,12 @@ enough:
   # presence, though.  NOTE: this means "[1.2.3.4]" IP addr HELO
   # strings, which are legit by RFC-2821, look like "!1.2.3.4!".
   # still useful though.
-  $ip =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
-  $rdns =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
-  $helo =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
-  $by =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
-  $ident =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
-  $envfrom =~ s/[\s\0\#\[\]\(\)\<\>\|]/!/gs;
+  $ip =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  $rdns =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  $helo =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  $by =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  $ident =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
+  $envfrom =~ s/[\s\000\#\[\]\(\)\<\>\|]/!/gs;
 
   my $relay = {
     ip => $ip,
@@ -1331,13 +1386,13 @@ sub make_relay_as_string {
 # spamcop does this, and it's a great idea ;)
 sub found_pop_fetcher_sig {
   my ($self) = @_;
-  if ($self->{allow_fetchmail_markers}) {
-    dbg("received-header: found fetchmail marker, restarting parse");
+  if ($self->{allow_mailfetch_markers}) {
+    dbg("received-header: found mail fetcher marker, restarting parse");
     $self->{relays_trusted} = [ ];
     $self->{relays_internal} = [ ];
     $self->{relays_external} = [ ];
   } else {
-    dbg("received-header: found fetchmail marker outside trusted area, ignored");
+    dbg("received-header: found mail fetcher marker outside trusted area, ignored");
   }
 }
 
