@@ -99,15 +99,12 @@ use Mail::SpamAssassin::Logger;
 
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
-use vars qw{
-  @ISA $VERSION
-};
-
-@ISA =                  qw();
-$VERSION =              'bogus';
+our @ISA = qw();
+#Removed $VERSION per BUG 6422
+#$VERSION =              'bogus';
 
 ###########################################################################
 
@@ -506,6 +503,10 @@ to add their own metadata as well.
 
 The C<Mail::SpamAssassin::Message> object for this message.
 
+=item permsgstatus
+
+The C<Mail::SpamAssassin::PerMsgStatus> context object for this scan.
+
 =back
 
 =item $plugin->parsed_metadata ( { options ... } )
@@ -676,7 +677,7 @@ format of:
 NOTE: This data structure has changed since it was originally introduced
 in version 3.0.0.  The values are no longer perl anonymous hashes, they
 are a single string containing the raw token value.  You can test for
-backwards compatability by checking to see if the value for a key is a
+backward compatibility by checking to see if the value for a key is a
 reference to a perl HASH, for instance:
 
 if (ref($toksref->{$sometokenkey}) eq 'HASH') {...
@@ -869,7 +870,7 @@ Indicate if the call is being made from a command line interface.
 
 =item $plugin->spamd_child_init ()
 
-Called when a new child starts up under spamd.
+Called in each new child process when it starts up under spamd.
 
 =item $plugin->log_scan_result ( { options ... } )
 
@@ -896,8 +897,6 @@ not be called.
 
 Called when the C<Mail::SpamAssassin> object is destroyed.
 
-=back
-
 =cut
 
 sub finish {
@@ -1005,6 +1004,8 @@ Set to 1 if warning messages should be suppressed.
 
 =back
 
+=back
+
 =head1 HELPER APIS
 
 These methods provide an API for plugins to register themselves
@@ -1079,6 +1080,8 @@ Tells the plugin handler to inhibit calling into other plugins in the plugin
 chain for the current callback.  Frequently used when parsing configuration
 settings using C<parse_config()>.
 
+=back
+
 =cut
 
 sub inhibit_further_callbacks {
@@ -1091,6 +1094,8 @@ __END__
 
 =head1 LOGGING
 
+=over 4
+
 =item Mail::SpamAssassin::Plugin::dbg($message)
 
 Output a debugging message C<$message>, if the SpamAssassin object is running
@@ -1188,7 +1193,7 @@ Different rule types receive different information by default:
 The configuration file arguments will be passed in after the standard
 arguments.
 
-=head1 BACKWARDS COMPATIBILITY
+=head1 BACKWARD COMPATIBILITY
 
 Note that if you write a plugin and need to determine if a particular
 helper method is supported on C<Mail::SpamAssassin::Plugin>, you
