@@ -23,11 +23,10 @@ use Mail::SpamAssassin::Constants qw(:sa);
 
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
-use vars qw(@ISA);
-@ISA = qw(Mail::SpamAssassin::Plugin);
+our @ISA = qw(Mail::SpamAssassin::Plugin);
 
 # constructor: register the eval rule
 sub new {
@@ -45,6 +44,7 @@ sub new {
   $self->register_eval_rule("check_blank_line_ratio");
   $self->register_eval_rule("tvd_vertical_words");
   $self->register_eval_rule("check_stock_info");
+  $self->register_eval_rule("check_body_length");
 
   return $self;
 }
@@ -119,7 +119,7 @@ sub _multipart_alternative_difference {
         }
 
 	# If there are no words, mark if there's at least 1 image ...
-	if (keys %html == 0 && exists $pms->{html}{inside}{img}) {
+	if (!%html && exists $pms->{html}{inside}{img}) {
 	  # Use "\n" as the mark since it can't ever occur normally
 	  $html{"\n"}=1;
 	}
@@ -217,6 +217,7 @@ sub tvd_vertical_words {
     }
   }
 
+  dbg("eval: tvd_vertical_words value: $pms->{tvd_vertical_words} / min: $min / max: $max - value must be >= min and < max");
   return 1 if ($pms->{tvd_vertical_words} >= $min && $pms->{tvd_vertical_words} < $max);
 }
 
@@ -282,5 +283,19 @@ sub _check_stock_info {
   return;
 }
 
+sub check_body_length {
+  my ($self, $pms, undef, $min) = @_;
+
+  my $body_length = $pms->{msg}->{pristine_body_length};
+  dbg("eval: body_length - %s - check for min of %s", $body_length, $min);
+
+  return (defined $body_length && $body_length <= $min) ? 1 : 0;
+}
+
+# ---------------------------------------------------------------------------
+
+# capability checks for "if can()":
+#
+sub has_check_body_length { 1 }
 
 1;
