@@ -1,5 +1,7 @@
 package Mail::SpamAssassin::Plugin::FreeMail;
-my $VERSION = 2.001;
+use strict;
+use warnings;
+my $VERSION = 2.002;
 
 ### About:
 #
@@ -80,6 +82,7 @@ my $VERSION = 2.001;
 # 1.999 - default whitelist undisclosed-recipient@yahoo.com etc
 # 2.000 - some cleaning up
 # 2.001 - fix freemail_whitelist
+# 2.002 - _add_desc -> _got_hit, fix description email append bug
 #
 
 ### Blah:
@@ -107,31 +110,11 @@ my $VERSION = 2.001;
 
 use Mail::SpamAssassin::Plugin;
 use Mail::SpamAssassin::PerMsgStatus;
-use Mail::SpamAssassin::Util::RegistrarBoundaries;
-use strict;
-use vars qw(@ISA);
-@ISA = qw(Mail::SpamAssassin::Plugin);
 
-# List of TLDs from RegistrarBoundaries.pm
-my $tlds = $Mail::SpamAssassin::Util::RegistrarBoundaries::VALID_TLDS_RE;
+our @ISA = qw(Mail::SpamAssassin::Plugin);
 
-### Some regexp tips courtesy of http://www.regular-expressions.info/email.html
-### v 0.02
-# full email regex
-my $email_regex = qr/
-  (?=.{0,64}\@)				# limit userpart to 64 chars (and speed up searching?)
-  (?<![a-z0-9!#$%&'*+\/=?^_`{|}~-])	# start boundary
-  (					# capture email
-  [a-z0-9!#$%&'*+\/=?^_`{|}~-]+		# no dot in beginning
-  (?:\.[a-z0-9!#$%&'*+\/=?^_`{|}~-]+)*	# no consecutive dots, no ending dot
-  \@
-  (?:[a-z0-9](?:[a-z0-9-]{0,59}[a-z0-9])?\.){1,4} # max 4x61 char parts (should be enough?)
-  ${tlds}				# ends with valid tld
-  )
-  (?!(?:[a-z0-9-]|\.[a-z0-9]))		# make sure domain ends here
-/xi;
 # default email whitelist
-my $email_whitelist = qr/
+our $email_whitelist = qr/
   ^(?:
       abuse|support|sales|info|helpdesk|contact|kontakt
     | (?:post|host|domain)master
@@ -145,7 +128,8 @@ my $email_whitelist = qr/
 
 # skip replyto check when envelope sender is
 # allow <> for now
-my $skip_replyto_envfrom = qr/
+# no re "strict";  # since perl 5.21.8: Ranges of ASCII printables...
+our $skip_replyto_envfrom = qr/
   (?:
       ^(?:post|host|domain)master
     | ^double-bounce
@@ -156,7 +140,6 @@ my $skip_replyto_envfrom = qr/
   )\@
 /xi;
 
-
 sub dbg { Mail::SpamAssassin::Plugin::dbg ("FreeMail: @_"); }
 
 sub new {
@@ -173,12 +156,28 @@ sub new {
     $self->register_eval_rule("check_freemail_header");
     $self->register_eval_rule("check_freemail_body");
 
+    # Need to init the regex here, utilizing registryboundaries->valid_tlds_re
+    # Some regexp tips courtesy of http://www.regular-expressions.info/email.html
+    # full email regex v0.02
+    $self->{email_regex} = qr/
+      (?=.{0,64}\@)				# limit userpart to 64 chars (and speed up searching?)
+      (?<![a-z0-9!#\$%&'*+\/=?^_`{|}~-])	# start boundary
+      (						# capture email
+      [a-z0-9!#\$%&'*+\/=?^_`{|}~-]+		# no dot in beginning
+      (?:\.[a-z0-9!#\$%&'*+\/=?^_`{|}~-]+)*	# no consecutive dots, no ending dot
+      \@
+      (?:[a-z0-9](?:[a-z0-9-]{0,59}[a-z0-9])?\.){1,4} # max 4x61 char parts (should be enough?)
+      $self->{main}->{registryboundaries}->{valid_tlds_re}	# ends with valid tld
+      )
+      (?!(?:[a-z0-9-]|\.[a-z0-9]))		# make sure domain ends here
+    /xi;
+
     return $self;
 }
 
 sub set_config {
     my ($self, $conf) = @_;
-    my @cmds = ();
+    my @cmds;
     push(@cmds, {
         setting => 'freemail_max_body_emails',
         default => 5,
@@ -328,7 +327,7 @@ sub _parse_body {
         my $parsed = $pms->get_uri_detail_list();
         while (my($uri, $info) = each %{$parsed}) {
             if (defined $info->{types}->{a} and not defined $info->{types}->{parsed}) {
-                if ($uri =~ /^mailto:${email_regex}/) {
+                if ($uri =~ /^(?:(?i)mailto):$self->{email_regex}/) {
                     my $email = lc($1);
                     push(@body_emails, $email) unless defined $seen{$email};
                     $seen{$email} = 1;
@@ -341,20 +340,21 @@ sub _parse_body {
         my $body = $pms->get_decoded_stripped_body_text_array();
         BODY: foreach (@$body) {
             # strip urls with possible emails inside
-            s#<?https?://\S{0,255}(?:\@|%40)\S{0,255}# #gi;
+            s{<?https?://\S{0,255}(?:\@|%40)\S{0,255}}{ }gi;
             # strip emails contained in <>, not mailto:
             # also strip ones followed by quote-like "wrote:" (but not fax: and tel: etc)
-            s#<?(?<!mailto:)${email_regex}(?:>|\s{1,10}(?!(?:fa(?:x|csi)|tel|phone|e?-?mail))[a-z]{2,11}:)# #gi;
-            while (/$email_regex/g) {
+            s{<?(?<!mailto:)$self->{email_regex}(?:>|\s{1,10}(?!(?:fa(?:x|csi)|tel|phone|e?-?mail))[a-z]{2,11}:)}{ }gi;
+            while (/$self->{email_regex}/g) {
                 my $email = lc($1);
-                push(@body_emails, $email) unless defined $seen{$email};
+                utf8::encode($email) if utf8::is_utf8($email); # chars to UTF-8
+                push(@body_emails, $email) unless $seen{$email};
                 $seen{$email} = 1;
-                last BODY if scalar @body_emails >= 40; # sanity
+                last BODY if @body_emails >= 40; # sanity
             }
         }
         my $count_all = 0;
         my $count_fm = 0;
-        foreach my $email (@body_emails) {
+        foreach my $email (@body_emails) {  # as UTF-8 octets
             if (++$count_all == $pms->{main}->{conf}->{freemail_max_body_emails}) {
                 if ($pms->{main}->{conf}->{freemail_skip_when_over_max}) {
                     $pms->{freemail_skip_body} = 1;
@@ -384,16 +384,21 @@ sub _parse_body {
     return 1;
 }
 
-sub _add_desc {
+sub _got_hit {
     my ($self, $pms, $email, $desc) = @_;
 
     my $rulename = $pms->get_current_eval_rule_name();
-    if (not defined $pms->{conf}->{descriptions}->{$rulename}) {
-        $pms->{conf}->{descriptions}->{$rulename} = $desc;
+
+    if (defined $pms->{conf}->{descriptions}->{$rulename}) {
+        $desc = $pms->{conf}->{descriptions}->{$rulename};
     }
+
     if ($pms->{main}->{conf}->{freemail_add_describe_email}) {
         $email =~ s/\@/[at]/g;
-        $pms->{conf}->{descriptions}->{$rulename} .= " ($email)";
+        $pms->got_hit($rulename, "", description => $desc." ($email)", ruletype => 'eval');
+    }
+    else {
+        $pms->got_hit($rulename, "", description => $desc, ruletype => 'eval');
     }
 }
 
@@ -419,24 +424,26 @@ sub check_freemail_header {
         }
     }
 
-    my $email = lc($pms->get(index($header,':') ? $header : $header.":addr"));
+    my @emails = map (lc, $pms->{main}->find_all_addrs_in_line ($pms->get($header)));
 
-    if ($email eq '') {
-        dbg("header $header not found from mail");
-        return 0;
+    if (!scalar (@emails)) {
+         dbg("header $header not found from mail");
+         return 0;
     }
-    dbg("address from header $header: $email");
+    dbg("addresses from header $header: ".join(';',@emails));
 
-    if ($self->_is_freemail($email)) {
-        if (defined $re) {
-            return 0 unless $email =~ $re;
-            dbg("HIT! $email is freemail and matches regex");
-        }
-        else {
-            dbg("HIT! $email is freemail");
-        }
-        $self->_add_desc($pms, $email, "Header $header is freemail");
-        return 1;
+    foreach my $email (@emails) {    
+        if ($self->_is_freemail($email)) {
+            if (defined $re) {
+                next unless $email =~ $re;
+                dbg("HIT! $email is freemail and matches regex");
+            }
+            else {
+                dbg("HIT! $email is freemail");
+            }
+            $self->_got_hit($pms, $email, "Header $header is freemail");
+            return 1;
+         }
     }
 
     return 0;
@@ -465,16 +472,16 @@ sub check_freemail_body {
         foreach my $email (keys %{$pms->{freemail_cache}{body}}) {
             if ($email =~ $re) {
                 dbg("HIT! email from body is freemail and matches regex: $email");
-                $self->_add_desc($pms, $email, "Email from body is freemail");
-                return 1;
+                $self->_got_hit($pms, $email, "Email from body is freemail");
+                return 0;
             }
         }
     }
     elsif (scalar keys %{$pms->{freemail_cache}{body}}) {
         my $emails = join(', ', keys %{$pms->{freemail_cache}{body}});
         dbg("HIT! body has freemails: $emails");
-        $self->_add_desc($pms, $emails, "Body contains freemails");
-        return 1;
+        $self->_got_hit($pms, $emails, "Body contains freemails");
+        return 0;
     }
 
     return 0;
@@ -516,8 +523,8 @@ sub check_freemail_from {
         else {
             dbg("HIT! $email is freemail");
         }
-        $self->_add_desc($pms, $email, "Sender address is freemail");
-        return 1;
+        $self->_got_hit($pms, $email, "Sender address is freemail");
+        return 0;
     }
 
     return 0;
@@ -560,8 +567,8 @@ sub check_freemail_replyto {
 
     if ($from_is_fm and $replyto_is_fm and ($from ne $replyto)) {
         dbg("HIT! From and Reply-To are different freemails");
-        $self->_add_desc($pms, "$from, $replyto", "From and Reply-To are different freemails");
-        return 1;
+        $self->_got_hit($pms, "$from, $replyto", "From and Reply-To are different freemails");
+        return 0;
     }
 
     if ($what eq 'replyto') {
@@ -591,8 +598,8 @@ sub check_freemail_replyto {
         foreach my $email (keys %{$pms->{freemail_cache}{body}}) {
             if ($email ne $check) {
                 dbg("HIT! $check and $email are different freemails");
-                $self->_add_desc($pms, "$check, $email", "Different freemails in reply header and body");
-                return 1;
+                $self->_got_hit($pms, "$check, $email", "Different freemails in reply header and body");
+                return 0;
             }
         }
     }
