@@ -38,11 +38,10 @@ use Mail::SpamAssassin::Logger;
 use Mail::SpamAssassin::Timeout;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
-use vars qw(@ISA);
-@ISA = qw(Mail::SpamAssassin::Plugin);
+our @ISA = qw(Mail::SpamAssassin::Plugin);
 
 # constructor: register the eval rule
 sub new {
@@ -59,11 +58,15 @@ sub new {
   $self->register_eval_rule ("check_for_spf_none");
   $self->register_eval_rule ("check_for_spf_fail");
   $self->register_eval_rule ("check_for_spf_softfail");
+  $self->register_eval_rule ("check_for_spf_permerror");
+  $self->register_eval_rule ("check_for_spf_temperror");
   $self->register_eval_rule ("check_for_spf_helo_pass");
   $self->register_eval_rule ("check_for_spf_helo_neutral");
   $self->register_eval_rule ("check_for_spf_helo_none");
   $self->register_eval_rule ("check_for_spf_helo_fail");
   $self->register_eval_rule ("check_for_spf_helo_softfail");
+  $self->register_eval_rule ("check_for_spf_helo_permerror");
+  $self->register_eval_rule ("check_for_spf_helo_temperror");
   $self->register_eval_rule ("check_for_spf_whitelist_from");
   $self->register_eval_rule ("check_for_def_spf_whitelist_from");
 
@@ -224,6 +227,15 @@ working downwards until results are successfully parsed.
   $conf->{parser}->register_commands(\@cmds);
 }
 
+
+=item has_check_for_spf_errors
+
+Adds capability check for "if can()" for check_for_spf_permerror, check_for_spf_temperror, check_for_spf_helo_permerror and check_for_spf_helo_permerror
+
+=cut 
+
+sub has_check_for_spf_errors { 1 }
+
 # SPF support
 sub check_for_spf_pass {
   my ($self, $scanner) = @_;
@@ -258,6 +270,18 @@ sub check_for_spf_softfail {
   $scanner->{spf_softfail};
 }
 
+sub check_for_spf_permerror {
+  my ($self, $scanner) = @_;
+  $self->_check_spf ($scanner, 0) unless $scanner->{spf_checked};
+  $scanner->{spf_permerror};
+}
+
+sub check_for_spf_temperror {
+  my ($self, $scanner) = @_;
+  $self->_check_spf ($scanner, 0) unless $scanner->{spf_checked};
+  $scanner->{spf_temperror};
+}
+
 sub check_for_spf_helo_pass {
   my ($self, $scanner) = @_;
   $self->_check_spf ($scanner, 1) unless $scanner->{spf_helo_checked};
@@ -291,6 +315,18 @@ sub check_for_spf_helo_softfail {
   $scanner->{spf_helo_softfail};
 }
 
+sub check_for_spf_helo_permerror {
+  my ($self, $scanner) = @_;
+  $self->_check_spf ($scanner, 1) unless $scanner->{spf_helo_checked};
+  $scanner->{spf_helo_permerror};
+}
+
+sub check_for_spf_helo_temperror {
+  my ($self, $scanner) = @_;
+  $self->_check_spf ($scanner, 1) unless $scanner->{spf_helo_checked};
+  $scanner->{spf_helo_temperror};
+}
+
 sub check_for_spf_whitelist_from {
   my ($self, $scanner) = @_;
   $self->_check_spf_whitelist($scanner) unless $scanner->{spf_whitelist_from_checked};
@@ -354,7 +390,7 @@ sub _check_spf {
 
 	# http://www.openspf.org/RFC_4408#header-field
 	# wtf - for some reason something is sticking an extra space between the header name and field value
-	if ($hdr =~ /^received-spf:\s*(pass|neutral|(?:soft)?fail|none)\b(?:.*\bidentity=(\S+?);?\b)?/i) {
+	if ($hdr =~ /^received-spf:\s*(pass|neutral|(?:soft)?fail|(?:temp|perm)error|none)\b(?:.*\bidentity=(\S+?);?\b)?/i) {
 	  my $result = lc($1);
 
 	  my $identity = '';	# we assume it's a mfrom check if we can't tell otherwise
@@ -381,6 +417,8 @@ sub _check_spf {
 	  $scanner->{"spf_${identity}none"} = 0;
 	  $scanner->{"spf_${identity}fail"} = 0;
 	  $scanner->{"spf_${identity}softfail"} = 0;
+	  $scanner->{"spf_${identity}temperror"} = 0;
+	  $scanner->{"spf_${identity}permerror"} = 0;
 	  $scanner->{"spf_${identity}failure_comment"} = undef;
 
 	  # and the result
@@ -402,7 +440,7 @@ sub _check_spf {
         # Authentication-Results: mail.example.com; SPF=none smtp.mailfrom=example.org (comment)
 
         my $tmphdr = $1;
-        if ($tmphdr =~ /^(pass|neutral|(?:hard|soft)?fail|none)(?:[^;]*?\bsmtp\.(\S+)\s*=[^;]+)?/i) {
+        if ($tmphdr =~ /^(pass|neutral|(?:hard|soft)?fail|(?:temp|perm)error|none)(?:[^;]*?\bsmtp\.(\S+)\s*=[^;]+)?/i) {
           my $result = lc($1);
           $result = 'fail'  if $result eq 'hardfail';  # RFC5451 permits this
 
@@ -430,6 +468,8 @@ sub _check_spf {
           $scanner->{"spf_${identity}none"} = 0;
           $scanner->{"spf_${identity}fail"} = 0;
           $scanner->{"spf_${identity}softfail"} = 0;
+          $scanner->{"spf_${identity}temperror"} = 0;
+          $scanner->{"spf_${identity}permerror"} = 0;
           $scanner->{"spf_${identity}failure_comment"} = undef;
 
           # and the result
@@ -468,7 +508,10 @@ sub _check_spf {
       # Mail::SPF::Server can be re-used, and we get to use our own resolver object!
       $self->{spf_server} = Mail::SPF::Server->new(
 				hostname     => $scanner->get_tag('HOSTNAME'),
-				dns_resolver => $self->{main}->{resolver} );
+				dns_resolver => $self->{main}->{resolver},
+				max_dns_interactive_terms => 20);
+      # Bug 7112: max_dns_interactive_terms defaults to 10, but even 14 is
+      # not enough for ebay.com, setting it to 15 NOTE: raising to 20 per bug 7182
       1;
     } or do {
       $eval_stat = $@ ne '' ? $@ : "errno=$!";  chomp $eval_stat;
@@ -524,6 +567,8 @@ sub _check_spf {
     $scanner->{spf_helo_none} = 0;
     $scanner->{spf_helo_fail} = 0;
     $scanner->{spf_helo_softfail} = 0;
+    $scanner->{spf_helo_permerror} = 0;
+    $scanner->{spf_helo_temperror} = 0;
     $scanner->{spf_helo_failure_comment} = undef;
   } else {
     # SPF on envelope sender (where possible)
@@ -533,6 +578,8 @@ sub _check_spf {
     $scanner->{spf_none} = 0;
     $scanner->{spf_fail} = 0;
     $scanner->{spf_softfail} = 0;
+    $scanner->{spf_permerror} = 0;
+    $scanner->{spf_temperror} = 0;
     $scanner->{spf_failure_comment} = undef;
   }
 
@@ -679,8 +726,11 @@ sub _check_spf {
     elsif ($result eq 'none') { $scanner->{spf_helo_none} = 1; }
     elsif ($result eq 'fail') { $scanner->{spf_helo_fail} = 1; }
     elsif ($result eq 'softfail') { $scanner->{spf_helo_softfail} = 1; }
+    elsif ($result eq 'permerror') { $scanner->{spf_helo_permerror} = 1; }
+    elsif ($result eq 'temperror') { $scanner->{spf_helo_temperror} = 1; }
+    elsif ($result eq 'error') { $scanner->{spf_helo_temperror} = 1; }
 
-    if ($result eq 'fail') {	# RFC 4408 6.2
+    if ($result eq 'fail') {	# RFC 7208 6.2
       $scanner->{spf_helo_failure_comment} = "SPF failed: $comment";
     }
   } else {
@@ -689,8 +739,11 @@ sub _check_spf {
     elsif ($result eq 'none') { $scanner->{spf_none} = 1; }
     elsif ($result eq 'fail') { $scanner->{spf_fail} = 1; }
     elsif ($result eq 'softfail') { $scanner->{spf_softfail} = 1; }
+    elsif ($result eq 'permerror') { $scanner->{spf_permerror} = 1; }
+    elsif ($result eq 'temperror') { $scanner->{spf_temperror} = 1; }
+    elsif ($result eq 'error') { $scanner->{spf_temperror} = 1; }
 
-    if ($result eq 'fail') {	# RCF 4408 6.2
+    if ($result eq 'fail') {	# RFC 7208 6.2
       $scanner->{spf_failure_comment} = "SPF failed: $comment";
     }
   }
