@@ -368,16 +368,11 @@ read_args(int argc, char **argv,
             case 'S':
             {
                 flags |= SPAMC_USE_SSL;
-		if (!spamc_optarg || (strcmp(spamc_optarg,"sslv3") == 0)) {
-		    flags |= SPAMC_SSLV3;
-		}
-		else if (strcmp(spamc_optarg,"tlsv1") == 0) {
-		    flags |= SPAMC_TLSV1;
-		}
-		else {
-		    libspamc_log(flags, LOG_ERR, "Please specify a legal ssl version (%s)", spamc_optarg);
-		    ret = EX_USAGE;
-		}
+                if(spamc_optarg) {
+                    libspamc_log(flags, LOG_ERR,
+                        "Explicit specification of an SSL/TLS version no longer supported.");
+                    ret = EX_USAGE;
+                }
                 break;
             }
 #endif
