@@ -12,6 +12,10 @@ use File::Basename;
 use File::Copy;
 use File::Path;
 use File::Spec;
+
+use Test::Builder ();
+use Test::More    ();
+
 use POSIX qw(WIFEXITED WIFSIGNALED WIFSTOPPED WEXITSTATUS WTERMSIG WSTOPSIG);
 
 use vars qw($RUNNING_ON_WINDOWS $SSL_AVAILABLE
@@ -338,10 +342,13 @@ sub sarun {
   $scrargs =~ s!/!\\!g if ($^O =~ /^MS(DOS|Win)/i);
   print ("\t$scrargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$scrargs > log/d.$testname/${Test::ntest} $post_redir");
+  
+  my $test_number = test_number();
+
+  system ("$scrargs > log/d.$testname/$test_number $post_redir");
   $sa_exitcode = ($?>>8);
   if ($sa_exitcode != 0) { return undef; }
-  &checkfile ("d.$testname/${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/$test_number", $read_sub) if (defined $read_sub);
   1;
 }
 
@@ -372,10 +379,13 @@ sub salearnrun {
   $salearnargs =~ s!/!\\!g if ($^O =~ /^MS(DOS|Win)/i);
   print ("\t$salearnargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$salearnargs > log/d.$testname/${Test::ntest}");
+
+  my $test_number = test_number();
+
+  system ("$salearnargs > log/d.$testname/$test_number");
   $salearn_exitcode = ($?>>8);
   if ($salearn_exitcode != 0) { return undef; }
-  &checkfile ("d.$testname/${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/$test_number", $read_sub) if (defined $read_sub);
   1;
 }
 
@@ -416,10 +426,13 @@ sub spamcrun {
 
   print ("\t$spamcargs\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
+
+  my $test_number = test_number();
+
   if ($capture_stderr) {
-    system ("$spamcargs > log/d.$testname/out.${Test::ntest} 2>&1");
+    system ("$spamcargs > log/d.$testname/out.$test_number 2>&1");
   } else {
-    system ("$spamcargs > log/d.$testname/out.${Test::ntest}");
+    system ("$spamcargs > log/d.$testname/out.$test_number");
   }
 
   $sa_exitcode = ($?>>8);
@@ -429,7 +442,7 @@ sub spamcrun {
 
   %found = ();
   %found_anti = ();
-  &checkfile ("d.$testname/out.${Test::ntest}", $read_sub) if (defined $read_sub);
+  &checkfile ("d.$testname/out.$test_number", $read_sub) if (defined $read_sub);
 
   if ($expect_failure) {
     ($sa_exitcode != 0);
@@ -459,7 +472,9 @@ sub spamcrun_background {
 
   print ("\t$spamcargs &\n");
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  system ("$spamcargs > log/d.$testname/bg.${Test::ntest} &") and return 0;
+  
+  my $test_number = test_number();
+  system ("$spamcargs > log/d.$testname/bg.$test_number &") and return 0;
 
   1;
 }
@@ -531,10 +546,12 @@ sub start_spamd {
   }
 
   (-d "log/d.$testname") or mkdir ("log/d.$testname", 0755);
-  my $spamd_stdout = "log/d.$testname/spamd.out.${Test::ntest}";
-     $spamd_stderr = "log/d.$testname/spamd.err.${Test::ntest}";    #  global
-  my $spamd_stdlog = "log/d.$testname/spamd.log.${Test::ntest}";
-
+  
+  my $test_number = test_number();
+  my $spamd_stdout = "log/d.$testname/spamd.out.$test_number";
+     $spamd_stderr = "log/d.$testname/spamd.err.$test_number";    #  global
+  my $spamd_stdlog = "log/d.$testname/spamd.log.$test_number";
+  my $spamd_pidfile = "log/spamd.pid";
   my $spamd_forker = $ENV{'SPAMD_FORKER'}   ?
                        $ENV{'SPAMD_FORKER'} :
                      $RUNNING_ON_WINDOWS    ?
@@ -550,36 +567,44 @@ sub start_spamd {
                        @spamd_args,
                        $spamd_extra_args,
                        qq{-s ${spamd_stderr}.timestamped},
+                       qq{-r ${spamd_pidfile}},
                        qq{&},
                     );
 
   # DEBUG instrumentation to trace spamd processes. See bug 5731 for history
   # if (-f "/home/jm/capture_spamd_straces") {
-  # $spamd_cmd = "strace -ttt -fo log/d.$testname/spamd.strace.${Test::ntest} $spamd_cmd";
+  # $spamd_cmd = "strace -ttt -fo log/d.$testname/spamd.strace.$test_number $spamd_cmd";
   # }
 
-  unlink ($spamd_stdout, $spamd_stderr, $spamd_stdlog);
+  unlink ($spamd_stdout, $spamd_stderr, $spamd_stdlog, $spamd_pidfile);
   print ("\t${spamd_cmd}\n");
   my $startat = time;
   system ($spamd_cmd);
 
-  # now find the PID
   $spamd_pid = 0;
+  # Find the PID, either in the pidfile or the log... 
   # note that the wait period increases the longer it takes,
   # 20 retries works out to a total of 60 seconds
   my $retries = 30;
-  my $wait = 0;
+  my $wait = 7;
+  sleep $wait ;
   while ($spamd_pid <= 0) {
     my $spamdlog = '';
-
+    my $pidstr = `cat $spamd_pidfile 2>/dev/null` ;
+    if ($pidstr) {
+       chomp $pidstr;
+       $spamd_pid = $pidstr;
+       dbgprint("Found PID $spamd_pid in pidfile\n");
+       last
+    }
     if (open (IN, "<${spamd_stderr}")) {
       while (<IN>) {
         # Yes, DO retry on this error. I'm getting test failures otherwise
         # /Address already in use/ and $retries = 0;
-	/server pid: (\d+)/ and $spamd_pid = $1;
+	/server pid: (\d+)/ and $spamd_pid = "$1" and dbgprint("Found PID $spamd_pid in stderr logfile\n");
 
-        if (/ERROR/) {
-          warn "spamd error! $_";
+        if ( !(/dbg: config: .*rulename/) && (/\bERROR/) ){
+          warn "spamd start failed - spamd error! $_\nExiting test with debug output";
           $retries = 0; last;
         }
 
@@ -589,12 +614,15 @@ sub start_spamd {
       last if ($spamd_pid);
     }
 
-    sleep (int($wait++ / 4) + 1) if $retries > 0;
+    my $sleep = (int($wait++ / 4) + 1);
+    warn "spam_pid not found: Sleeping $sleep - Retry # $retries\n";
+
+    sleep $sleep if $retries > 0;
 
     if ($retries-- <= 0) {
-      warn "spamd start failed: log: $spamdlog";
+      warn "spamd start failed - Could not find a valid PID.\nEnd Debug log -------------------\n$spamdlog\nEnd Debug log -------------------";
       warn "\n\nMaybe you need to kill a running spamd process?\n";
-      warn "started at $startat, gave up at ".time."\n\n";
+      warn "Or the start took too long. Started at $startat, gave up at ".time."\n\n";
       return 0;
     }
   }
@@ -993,7 +1021,7 @@ sub system_or_die {
 
 # (sub exit_status_str copied from Util.pm)
 # map process termination status number to an informative string, and
-# append optional mesage (dual-valued errno or a string or a number),
+# append optional message (dual-valued errno or a string or a number),
 # returning the resulting string
 #
 sub exit_status_str($;$) {
@@ -1027,7 +1055,7 @@ sub can_use_net_dns_safely {
   # (which is used by Net::DNS)
 
   return 1 if ($< != 0);
-  return 1 if ($^O =~ /^(linux|mswin|dos|os2)/oi);
+  return 1 if ($^O =~ /^(linux|mswin|dos|os2|openbsd)/oi);
 
   my $has_unsafe_hostname =
     eval { require Sys::Hostname::Long && Sys::Hostname::Long->VERSION < 1.4 };
@@ -1072,4 +1100,8 @@ sub debug_array {
   return $string;
 }
 
+sub test_number {
+  return Test::More->builder->current_test;
+}
+
 1;
